/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/* 
 * imsettings-stat.c
 * Copyright (C) 2011 Red Hat, Inc. All rights reserved.
 * 
 * Authors:
 *   Akira TAGOH  <tagoh@redhat.com>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <glib.h>
#include <glib/gi18n.h>
#include <gio/gio.h>
#include <X11/Xlib.h>
#include "imsettings.h"
#include "imsettings-client.h"

/*< private >*/

/*< public >*/

int
main(int    argc,
     char **argv)
{
	gboolean arg_detail = FALSE;
	GOptionContext *ctx = g_option_context_new(NULL);
	GOptionEntry entries[] = {
		/* For translators: this is a translation for the command-line option. */
		{"detail", 'd', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_NONE, &arg_detail, N_("Output the detail information for the result"), NULL},
		{NULL, 0, 0, 0, NULL, NULL, NULL}
	};
	GError *error = NULL;
	gboolean is_dbus_running = TRUE, is_xsettings_running = TRUE, has_consistent = TRUE;
	gchar *reason_dbus = NULL, *reason_xsettings = NULL, *reason_consistent = NULL;
	GString *current_settings = g_string_new(NULL);

#ifdef ENABLE_NLS
	bindtextdomain (GETTEXT_PACKAGE, IMSETTINGS_LOCALEDIR);
#ifdef HAVE_BIND_TEXTDOMAIN_CODESET
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
#endif /* HAVE_BIND_TEXTDOMAIN_CODESET */
	textdomain (GETTEXT_PACKAGE);
#endif /* ENABLE_NLS */

	setlocale(LC_ALL, "");

	g_type_init();

	g_option_context_add_main_entries(ctx, entries, GETTEXT_PACKAGE);
	if (!g_option_context_parse(ctx, &argc, &argv, &error)) {
		if (error != NULL) {
			g_printerr("%s\n", error->message);
		} else {
			g_warning(_("Unknown error in parsing the command lines."));
		}
		return 1;
	}

	/* check if DBus is running */
	G_STMT_START {
		GDBusConnection *connection;

		connection = g_bus_get_sync(G_BUS_TYPE_SESSION,
					    NULL,
					    &error);
		if (error) {
			reason_dbus = g_strdup(error->message);
			g_error_free(error);
			is_dbus_running = FALSE;
		}
		if (connection)
			g_object_unref(connection);
		if (!is_dbus_running)
			goto result;
	} G_STMT_END;

	/* check if XSETTINGS manager is running */
	G_STMT_START {
		Display *dpy = XOpenDisplay(NULL);
		char buffer[256];
		Atom selection_atom;

		if (!dpy) {
			reason_xsettings = g_strdup(_("Unable to open X display"));
			is_xsettings_running = FALSE;
			goto result;
		}
		sprintf(buffer, "_XSETTINGS_S%d", DefaultScreen(dpy));
		selection_atom = XInternAtom(dpy, buffer, False);
		if (!XGetSelectionOwner(dpy, selection_atom)) {
			reason_xsettings = g_strdup(_("XSETTINGS manager isn't running"));
			is_xsettings_running = FALSE;
		}
		XCloseDisplay(dpy);
		if (!is_xsettings_running)
			goto result;
	} G_STMT_END;

	/* check current settings */
	G_STMT_START {
		gchar *locale = setlocale(LC_CTYPE, NULL);
		IMSettingsClient *client = imsettings_client_new(locale);
		GVariant *v = NULL;
		GVariantIter *iter;
		const gchar *key, *val, *last_val = NULL;
		gboolean no_cmp = TRUE;

		if (!client) {
			reason_consistent = g_strdup(_("Unable to create a client instance."));
			has_consistent = FALSE;
			goto result;
		}
		if (imsettings_client_get_version(client, NULL, &error) != IMSETTINGS_SETTINGS_API_VERSION) {
			if (error) {
				reason_consistent = g_strdup(error->message);
				has_consistent = FALSE;
			} else {
				reason_consistent = g_strdup(_("imsettings version mismatch"));
				has_consistent = FALSE;
			}
			goto finalize_client;
		}
		v = imsettings_client_get_module_settings(client, NULL, &error);
		if (error) {
			reason_consistent = g_strdup(error->message);
			has_consistent = FALSE;
			goto finalize_client;
		}
		g_variant_get(v, "a{ss}", &iter);
		while (g_variant_iter_next(iter, "{&s&s}", &key, &val)) {
			if (no_cmp) {
				last_val = val;
				no_cmp = FALSE;
			} else {
				if (has_consistent && g_strcmp0(last_val, val) != 0) {
					has_consistent = FALSE;
					reason_consistent = g_strdup(_("Please see .imsettings.log for more details"));
				}
				last_val = val;
			}
			g_string_append_printf(current_settings, "    %s:\t%s\n", key, val);
		}
		g_variant_iter_free(iter);
		g_variant_unref(v);
	  finalize_client:
		if (client)
			g_object_unref(client);
	} G_STMT_END;

  result:
	if (arg_detail) {
		g_print("DBus availability: %s\n", is_dbus_running ? "true" : "false");
		if (!is_dbus_running) {
			g_print("  reason: %s\n", reason_dbus);
			goto end;
		}
		g_print("XSETTINGS avaiiability: %s\n", is_xsettings_running ? "true" : "false");
		if (!is_xsettings_running) {
			g_print("  reason: %s\n", reason_xsettings);
			goto end;
		}
		g_print("Current settings in modules: %s\n", has_consistent ? "consistent" : "inconsistent");
		if (reason_consistent)
			g_print("  reason: %s\n", reason_consistent);
		g_print("  result:\n%s", current_settings->str);
	}
  end:
	g_string_free(current_settings, TRUE);
	g_free(reason_dbus);
	g_free(reason_xsettings);
	g_free(reason_consistent);

	return is_dbus_running && is_xsettings_running && has_consistent ? 0 : -1;
}
